﻿using MedicalDatabaseContracts;
using MedicalDatabaseContracts.Models;
using MedicalDatabaseContracts.SearchModels;
using MedicalDatabaseContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System.Windows.Forms;

namespace MedicalView.Visits
{
    public partial class FormVisit : FormAbstractDetail
    {
        private readonly ILogger _logger;
        private readonly ILogic<Visit, VisitViewModel, VisitSearchModel> _visitLogic;
        private readonly ILogic<Diagnose, DiagnoseViewModel, DiagnoseSearchModel> _diagnoseLogic;
        private readonly ILogic<Doctor, DoctorViewModel, DoctorSearchModel> _doctorLogic;
        private readonly ILogic<Patient, PatientViewModel, PatientSearchModel> _patientLogic;
        public FormVisit(
            ILogger<FormVisit> logger,
            ILogic<Visit, VisitViewModel, VisitSearchModel> visitLogic,
            ILogic<Diagnose, DiagnoseViewModel, DiagnoseSearchModel> diagnoseLogic,
            ILogic<Doctor, DoctorViewModel, DoctorSearchModel> doctorLogic,
            ILogic<Patient, PatientViewModel, PatientSearchModel> patientLogic)
        {
            InitializeComponent();
            _logger = logger;
            _visitLogic = visitLogic;
            _diagnoseLogic = diagnoseLogic;
            _doctorLogic = doctorLogic;
            _patientLogic = patientLogic;
        }

        private void ApplyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (comboBoxPatient.SelectedItem == null)
            {
                MessageBox.Show("Выберите пациента", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            else if (comboBoxDoctor.SelectedItem == null)
            {
                MessageBox.Show("Выберите врача", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Сохранение модели приема");
            try
            {
                var model = new Visit
                {
                    Id = ModelId ?? 0,
                    Comment = textBoxComment.Text,
                    PatientId = (comboBoxPatient.SelectedItem as PatientViewModel).Id,
                    DoctorId = (comboBoxDoctor.SelectedItem as DoctorViewModel).Id,
                    DiagnoseId = comboBoxDiagnose.SelectedItem != null ? (comboBoxDiagnose.SelectedItem as DiagnoseViewModel).Id : null,
                    Date = DateOnly.FromDateTime(datePicker.Value),
                    Time = TimeOnly.FromDateTime(timePicker.Value)
                };
                var operationResult = ModelId.HasValue ? _visitLogic.Update(model) : _visitLogic.Create(model);
                if (!operationResult)
                {
                    throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                }
                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка сохранения модели приема");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void CancelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Cancel();
        }

        private void FormVisit_Load(object sender, EventArgs e)
        {
            if (ModelId.HasValue)
            {
                try
                {
                    _logger.LogInformation("Получение модели приема");
                    var view = _visitLogic.ReadElement(ModelId.Value);
                    if (view != null)
                    {
                        textBoxComment.Text = view.Comment;
                        datePicker.Value = view.Date.ToDateTime(view.Time);
                        timePicker.Value = view.Date.ToDateTime(view.Time);
                        var patients = _patientLogic.ReadList();
                        var doctors = _doctorLogic.ReadList();
                        var diagnoses = _diagnoseLogic.ReadList();
                        comboBoxPatient.DataSource = patients;
                        comboBoxDoctor.DataSource = doctors;
                        comboBoxDiagnose.DataSource = diagnoses;
                        if (view.DiagnoseId != null)
                        {
                            comboBoxDiagnose.SelectedIndex = diagnoses.FindIndex(x => x.Id == view.DiagnoseId);
                        }
                        comboBoxDoctor.SelectedIndex = doctors.FindIndex(x => x.Id == view.DoctorId);
                        comboBoxPatient.SelectedIndex = patients.FindIndex(x => x.Id == view.PatientId);
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка получения модели приема");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
    }
}
